/* Copyright 2013 Andrey Dubovik <andrei.dubovik@gmail.com>
 *
 * This file is part of Minido.
 *
 * Minido is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Minido is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Minido. If not, see <http://www.gnu.org/licenses/>.
 */

#include "applications.hpp"
#include "unicode.hpp"
#include <fstream>
#include <vector>
#include <sys/types.h>
#include <dirent.h>

static Collection* collection;
static DataBase* database;

using namespace std;

__attribute__((constructor))
void constructor() {
  collection = new Collection;
}

__attribute__((destructor))

void destructor() {
  delete collection;
}

Application::Application(DataBase &db): Item(db) {}

void Application::invoke() {
  system((exec + "&").c_str());
}

void populate(DataBase& db) {
  database = &db;
  string home(getenv("HOME"));

  loadDir("/usr/share/applications/");
  loadDir("/usr/local/share/applications/");
  loadDir(home + "/.local/share/applications/");
}

void loadFile(const string& filename) {
  ifstream file(filename.c_str());

  string type;
  string name;
  string exec;
  string icon;

  string line;
  while(getline(file, line)) {
    if(!line.size()) continue;
    if(line[0] == '#') continue;
    if(line[0] == '[') {
      if(line != "[Desktop Entry]") break;
    }
    else {
      auto cur = line.find('=');
      string key = line.substr(0, cur);
      if(key == "Type") {
	type = line.substr(cur+1);
	continue;
      }
      if(key == "Name") {
	name = line.substr(cur+1);
	continue;
      }
      if(key == "Exec") {
	exec = line.substr(cur+1);
	continue;
      }
      if(key == "Icon") {
	icon = line.substr(cur+1);
	continue;
      }
    }
  }
  
  if(type == "Application" && name.size()) {
    auto& app = (*collection)[name];
    if(!app) {
      app.reset(new Application(*database));
      app->setName(utf8to32(name));
    }
    
    if(icon.size() >= 4) {
      string ext = icon.substr(icon.size()-4);
      if(ext == ".png" || ext == ".svg") {
	icon = icon.substr(0, icon.size()-4);
      }
    }
    if(icon.size()) app->icon = Icon(icon);
    if(exec.size()) app->exec = parseExec(exec);
  }  
}

// TODO: Implement full specification?
// Current functionality:
//   * recognize '\' as an escape character,
//   * remove arguments starting with '%'.
std::string parseExec(const std::string& str) {
  vector<string> args;

  args.push_back(string());
  string* arg = &args.back();

  for(int i = 0; i < str.size(); ++i) {
    if(str[i] == '\\' && i < str.size()-1) {
      arg->push_back(str[++i]);
      continue;
    }
    if(str[i] == ' ') {
      if(arg->size()) {
	args.push_back(string());
	arg = &args.back();
      }
      continue;
    }
    arg->push_back(str[i]);
  }

  string newstr;
  for(auto& arg: args) {
    if(!arg.size()) continue;
    if(arg[0] == '%') continue;
    newstr += arg;
    newstr += ' ';
  }

  if(newstr.size()) newstr.pop_back();
  return newstr;
}

void loadDir(const string& dirname) {
  DIR* dir = opendir(dirname.c_str());
  if(!dir) return;

  while(struct dirent* entry = readdir(dir)) {
    string name(entry->d_name);
    if(entry->d_type == DT_REG) {
      if(name.size() >= 8 && name.substr(name.size()-8) == ".desktop") {
	loadFile(dirname + name);
      }
    }
    if(entry->d_type == DT_DIR) {
      if(name == ".") continue;
      if(name == "..") continue;
      loadDir(dirname + name + "/");
    }
  }
  closedir(dir);
}
